@@ -19,7 +19,6 @@ package app
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"net/http/httptest"
@@ -43,12 +42,11 @@ import (
 	"k8s.io/kubernetes/pkg/scheduler/apis/config"
 	"k8s.io/kubernetes/pkg/scheduler/apis/config/testing/defaults"
 	"k8s.io/kubernetes/pkg/scheduler/framework"
-	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/names"
 )
 
 func TestSetup(t *testing.T) {
 	// temp dir
-	tmpDir, err := ioutil.TempDir("", "scheduler-options")
+	tmpDir, err := os.MkdirTemp("", "scheduler-options")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -62,7 +60,7 @@ func TestSetup(t *testing.T) {
 	defer server.Close()
 
 	configKubeconfig := filepath.Join(tmpDir, "config.kubeconfig")
-	if err := ioutil.WriteFile(configKubeconfig, []byte(fmt.Sprintf(`
+	if err := os.WriteFile(configKubeconfig, []byte(fmt.Sprintf(`
 apiVersion: v1
 kind: Config
 clusters:
@@ -85,16 +83,17 @@ users:
 	}
 
 	// plugin config
-	pluginConfigFilev1beta3 := filepath.Join(tmpDir, "pluginv1beta3.yaml")
-	if err := ioutil.WriteFile(pluginConfigFilev1beta3, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
+	pluginConfigFilev1 := filepath.Join(tmpDir, "pluginv1.yaml")
+	if err := os.WriteFile(pluginConfigFilev1, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
 profiles:
 - plugins:
     multiPoint:
       enabled:
+      - name: SchedulingGates
       - name: DefaultBinder
       - name: PrioritySort
       - name: DefaultPreemption
@@ -123,38 +122,59 @@ profiles:
 	}
 
 	// plugin config
-	pluginConfigFilev1beta2 := filepath.Join(tmpDir, "pluginv1beta2.yaml")
-	if err := ioutil.WriteFile(pluginConfigFilev1beta2, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta2
+	pluginConfigFilev1beta3 := filepath.Join(tmpDir, "pluginv1beta3.yaml")
+	if err := os.WriteFile(pluginConfigFilev1beta3, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1beta3
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
+profiles:
+- plugins:
+    multiPoint:
+      enabled:
+      - name: SchedulingGates
+      - name: DefaultBinder
+      - name: PrioritySort
+      - name: DefaultPreemption
+      - name: VolumeBinding
+      - name: NodeResourcesFit
+      - name: NodePorts
+      - name: InterPodAffinity
+      - name: TaintToleration
+      disabled:
+      - name: "*"
+    preFilter:
+      disabled:
+      - name: VolumeBinding
+      - name: InterPodAffinity
+    filter:
+      disabled:
+      - name: VolumeBinding
+      - name: InterPodAffinity
+      - name: TaintToleration
+    score:
+      disabled:
+      - name: VolumeBinding
+      - name: NodeResourcesFit
+`, configKubeconfig)), os.FileMode(0600)); err != nil {
+		t.Fatal(err)
+	}
+
+	// out-of-tree plugin config v1
+	outOfTreePluginConfigFilev1 := filepath.Join(tmpDir, "outOfTreePluginv1.yaml")
+	if err := os.WriteFile(outOfTreePluginConfigFilev1, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1
+kind: KubeSchedulerConfiguration
+clientConnection:
+  kubeconfig: '%s'
 profiles:
 - plugins:
     preFilter:
       enabled:
-      - name: NodeResourcesFit
-      - name: NodePorts
-      disabled:
-      - name: "*"
+      - name: Foo
     filter:
       enabled:
-      - name: NodeResourcesFit
-      - name: NodePorts
-      disabled:
-      - name: "*"
-    preScore:
-      enabled:
-      - name: InterPodAffinity
-      - name: TaintToleration
-      disabled:
-      - name: "*"
-    score:
-      enabled:
-      - name: InterPodAffinity
-      - name: TaintToleration
-      disabled:
-      - name: "*"
+      - name: Foo
 `, configKubeconfig)), os.FileMode(0600)); err != nil {
 		t.Fatal(err)
 	}
@@ -165,26 +185,7 @@ profiles:
 apiVersion: kubescheduler.config.k8s.io/v1beta3
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
-profiles:
-- plugins:
-    preFilter:
-      enabled:
-      - name: Foo
-    filter:
-      enabled:
-      - name: Foo
-`, configKubeconfig)), os.FileMode(0600)); err != nil {
-		t.Fatal(err)
-	}
-
-	// out-of-tree plugin config v1beta2
-	outOfTreePluginConfigFilev1beta2 := filepath.Join(tmpDir, "outOfTreePluginv1beta2.yaml")
-	if err := os.WriteFile(outOfTreePluginConfigFilev1beta2, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta2
-kind: KubeSchedulerConfiguration
-clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
 profiles:
 - plugins:
     preFilter:
@@ -199,11 +200,11 @@ profiles:
 
 	// multiple profiles config
 	multiProfilesConfig := filepath.Join(tmpDir, "multi-profiles.yaml")
-	if err := ioutil.WriteFile(multiProfilesConfig, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta2
+	if err := os.WriteFile(multiProfilesConfig, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
 profiles:
 - schedulerName: "profile-default-plugins"
 - schedulerName: "profile-disable-all-filter-and-score-plugins"
@@ -229,22 +230,22 @@ profiles:
 
 	// empty leader-election config
 	emptyLeaderElectionConfig := filepath.Join(tmpDir, "empty-leader-election-config.yaml")
-	if err := ioutil.WriteFile(emptyLeaderElectionConfig, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
+	if err := os.WriteFile(emptyLeaderElectionConfig, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
 `, configKubeconfig)), os.FileMode(0600)); err != nil {
 		t.Fatal(err)
 	}
 
 	// leader-election config
 	leaderElectionConfig := filepath.Join(tmpDir, "leader-election-config.yaml")
-	if err := ioutil.WriteFile(leaderElectionConfig, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
+	if err := os.WriteFile(leaderElectionConfig, []byte(fmt.Sprintf(`
+apiVersion: kubescheduler.config.k8s.io/v1
 kind: KubeSchedulerConfiguration
 clientConnection:
-  kubeconfig: "%s"
+  kubeconfig: '%s'
 leaderElection:
   leaseDuration: 1h
 `, configKubeconfig)), os.FileMode(0600)); err != nil {
@@ -252,43 +253,42 @@ leaderElection:
 	}
 
 	testcases := []struct {
-		name               string
-		flags              []string
-		registryOptions    []Option
-		restoreFeatures    map[featuregate.Feature]bool
-		wantPlugins        map[string]*config.Plugins
-		wantLeaderElection *componentbaseconfig.LeaderElectionConfiguration
+		name                 string
+		flags                []string
+		registryOptions      []Option
+		restoreFeatures      map[featuregate.Feature]bool
+		wantPlugins          map[string]*config.Plugins
+		wantLeaderElection   *componentbaseconfig.LeaderElectionConfiguration
+		wantClientConnection *componentbaseconfig.ClientConnectionConfiguration
 	}{
 		{
-			name: "default config with an alpha feature enabled and an beta feature disabled",
+			name: "default config with an alpha feature enabled",
 			flags: []string{
 				"--kubeconfig", configKubeconfig,
-				"--feature-gates=VolumeCapacityPriority=true,DefaultPodTopologySpread=false",
+				"--feature-gates=VolumeCapacityPriority=true",
+			},
+			wantPlugins: map[string]*config.Plugins{
+				"default-scheduler": defaults.ExpandedPluginsV1,
+			},
+			restoreFeatures: map[featuregate.Feature]bool{
+				features.VolumeCapacityPriority: false,
+			},
+		},
+		{
+			name: "default config with a beta feature disabled",
+			flags: []string{
+				"--kubeconfig", configKubeconfig,
+				"--feature-gates=PodSchedulingReadiness=false",
 			},
 			wantPlugins: map[string]*config.Plugins{
 				"default-scheduler": func() *config.Plugins {
-					plugins := &config.Plugins{
-						QueueSort:  defaults.ExpandedPluginsV1beta3.QueueSort,
-						PreFilter:  defaults.ExpandedPluginsV1beta3.PreFilter,
-						Filter:     defaults.ExpandedPluginsV1beta3.Filter,
-						PostFilter: defaults.ExpandedPluginsV1beta3.PostFilter,
-						PreScore:   defaults.ExpandedPluginsV1beta3.PreScore,
-						Score:      defaults.ExpandedPluginsV1beta3.Score,
-						Bind:       defaults.ExpandedPluginsV1beta3.Bind,
-						PreBind:    defaults.ExpandedPluginsV1beta3.PreBind,
-						Reserve:    defaults.ExpandedPluginsV1beta3.Reserve,
-					}
-					plugins.PreScore.Enabled = append(plugins.PreScore.Enabled, config.Plugin{Name: names.SelectorSpread, Weight: 0})
-					plugins.Score.Enabled = append(
-						plugins.Score.Enabled,
-						config.Plugin{Name: names.SelectorSpread, Weight: 1},
-					)
+					plugins := defaults.ExpandedPluginsV1.DeepCopy()
+					plugins.PreEnqueue = config.PluginSet{}
 					return plugins
 				}(),
 			},
 			restoreFeatures: map[featuregate.Feature]bool{
-				features.VolumeCapacityPriority:   false,
-				features.DefaultPodTopologySpread: true,
+				features.PodSchedulingReadiness: true,
 			},
 		},
 		{
@@ -297,47 +297,7 @@ leaderElection:
 				"--kubeconfig", configKubeconfig,
 			},
 			wantPlugins: map[string]*config.Plugins{
-				"default-scheduler": defaults.ExpandedPluginsV1beta3,
-			},
-		},
-		{
-			name: "component configuration v1beta2",
-			flags: []string{
-				"--config", pluginConfigFilev1beta2,
-				"--kubeconfig", configKubeconfig,
-			},
-			wantPlugins: map[string]*config.Plugins{
-				"default-scheduler": {
-					Bind: config.PluginSet{Enabled: []config.Plugin{{Name: "DefaultBinder"}}},
-					Filter: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "NodeResourcesFit"},
-							{Name: "NodePorts"},
-						},
-					},
-					PreFilter: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "NodeResourcesFit"},
-							{Name: "NodePorts"},
-						},
-					},
-					PostFilter: config.PluginSet{Enabled: []config.Plugin{{Name: "DefaultPreemption"}}},
-					PreScore: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "InterPodAffinity"},
-							{Name: "TaintToleration"},
-						},
-					},
-					QueueSort: config.PluginSet{Enabled: []config.Plugin{{Name: "PrioritySort"}}},
-					Score: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "InterPodAffinity", Weight: 1},
-							{Name: "TaintToleration", Weight: 1},
-						},
-					},
-					Reserve: config.PluginSet{Enabled: []config.Plugin{{Name: "VolumeBinding"}}},
-					PreBind: config.PluginSet{Enabled: []config.Plugin{{Name: "VolumeBinding"}}},
-				},
+				"default-scheduler": defaults.ExpandedPluginsV1,
 			},
 		},
 		{
@@ -347,62 +307,57 @@ leaderElection:
 				"--kubeconfig", configKubeconfig,
 			},
 			wantPlugins: map[string]*config.Plugins{
-				"default-scheduler": {
-					Bind: config.PluginSet{Enabled: []config.Plugin{{Name: "DefaultBinder"}}},
-					Filter: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "NodeResourcesFit"},
-							{Name: "NodePorts"},
-						},
-					},
-					PreFilter: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "NodeResourcesFit"},
-							{Name: "NodePorts"},
-						},
-					},
-					PostFilter: config.PluginSet{Enabled: []config.Plugin{{Name: "DefaultPreemption"}}},
-					PreScore: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "InterPodAffinity"},
-							{Name: "TaintToleration"},
-						},
-					},
-					QueueSort: config.PluginSet{Enabled: []config.Plugin{{Name: "PrioritySort"}}},
-					Score: config.PluginSet{
-						Enabled: []config.Plugin{
-							{Name: "InterPodAffinity", Weight: 1},
-							{Name: "TaintToleration", Weight: 1},
-						},
-					},
-					Reserve: config.PluginSet{Enabled: []config.Plugin{{Name: "VolumeBinding"}}},
-					PreBind: config.PluginSet{Enabled: []config.Plugin{{Name: "VolumeBinding"}}},
-				},
+				"default-scheduler": func() *config.Plugins {
+					plugins := defaults.ExpandedPluginsV1beta3.DeepCopy()
+					plugins.Filter.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "NodePorts"},
+					}
+					plugins.PreFilter.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "NodePorts"},
+					}
+					plugins.PreScore.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "InterPodAffinity"},
+						{Name: "TaintToleration"},
+					}
+					plugins.Score.Enabled = []config.Plugin{
+						{Name: "InterPodAffinity", Weight: 1},
+						{Name: "TaintToleration", Weight: 1},
+					}
+					return plugins
+				}(),
 			},
 		},
 		{
-			name: "out-of-tree component configuration v1beta2",
+			name: "component configuration v1",
 			flags: []string{
-				"--config", outOfTreePluginConfigFilev1beta2,
+				"--config", pluginConfigFilev1,
 				"--kubeconfig", configKubeconfig,
 			},
-			registryOptions: []Option{WithPlugin("Foo", newFoo)},
 			wantPlugins: map[string]*config.Plugins{
-				"default-scheduler": {
-					Bind: defaults.PluginsV1beta2.Bind,
-					Filter: config.PluginSet{
-						Enabled: append(defaults.PluginsV1beta2.Filter.Enabled, config.Plugin{Name: "Foo"}),
-					},
-					PreFilter: config.PluginSet{
-						Enabled: append(defaults.PluginsV1beta2.PreFilter.Enabled, config.Plugin{Name: "Foo"}),
-					},
-					PostFilter: defaults.PluginsV1beta2.PostFilter,
-					PreScore:   defaults.PluginsV1beta2.PreScore,
-					QueueSort:  defaults.PluginsV1beta2.QueueSort,
-					Score:      defaults.PluginsV1beta2.Score,
-					Reserve:    defaults.PluginsV1beta2.Reserve,
-					PreBind:    defaults.PluginsV1beta2.PreBind,
-				},
+				"default-scheduler": func() *config.Plugins {
+					plugins := defaults.ExpandedPluginsV1.DeepCopy()
+					plugins.Filter.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "NodePorts"},
+					}
+					plugins.PreFilter.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "NodePorts"},
+					}
+					plugins.PreScore.Enabled = []config.Plugin{
+						{Name: "NodeResourcesFit"},
+						{Name: "InterPodAffinity"},
+						{Name: "TaintToleration"},
+					}
+					plugins.Score.Enabled = []config.Plugin{
+						{Name: "InterPodAffinity", Weight: 1},
+						{Name: "TaintToleration", Weight: 1},
+					}
+					return plugins
+				}(),
 			},
 		},
 		{
@@ -413,21 +368,28 @@ leaderElection:
 			},
 			registryOptions: []Option{WithPlugin("Foo", newFoo)},
 			wantPlugins: map[string]*config.Plugins{
-				"default-scheduler": {
-					Bind: defaults.ExpandedPluginsV1beta3.Bind,
-					Filter: config.PluginSet{
-						Enabled: append(defaults.ExpandedPluginsV1beta3.Filter.Enabled, config.Plugin{Name: "Foo"}),
-					},
-					PreFilter: config.PluginSet{
-						Enabled: append(defaults.ExpandedPluginsV1beta3.PreFilter.Enabled, config.Plugin{Name: "Foo"}),
-					},
-					PostFilter: defaults.ExpandedPluginsV1beta3.PostFilter,
-					PreScore:   defaults.ExpandedPluginsV1beta3.PreScore,
-					QueueSort:  defaults.ExpandedPluginsV1beta3.QueueSort,
-					Score:      defaults.ExpandedPluginsV1beta3.Score,
-					Reserve:    defaults.ExpandedPluginsV1beta3.Reserve,
-					PreBind:    defaults.ExpandedPluginsV1beta3.PreBind,
-				},
+				"default-scheduler": func() *config.Plugins {
+					plugins := defaults.ExpandedPluginsV1beta3.DeepCopy()
+					plugins.PreFilter.Enabled = append(plugins.PreFilter.Enabled, config.Plugin{Name: "Foo"})
+					plugins.Filter.Enabled = append(plugins.Filter.Enabled, config.Plugin{Name: "Foo"})
+					return plugins
+				}(),
+			},
+		},
+		{
+			name: "out-of-tree component configuration v1",
+			flags: []string{
+				"--config", outOfTreePluginConfigFilev1,
+				"--kubeconfig", configKubeconfig,
+			},
+			registryOptions: []Option{WithPlugin("Foo", newFoo)},
+			wantPlugins: map[string]*config.Plugins{
+				"default-scheduler": func() *config.Plugins {
+					plugins := defaults.ExpandedPluginsV1.DeepCopy()
+					plugins.PreFilter.Enabled = append(plugins.PreFilter.Enabled, config.Plugin{Name: "Foo"})
+					plugins.Filter.Enabled = append(plugins.Filter.Enabled, config.Plugin{Name: "Foo"})
+					return plugins
+				}(),
 			},
 		},
 		{
@@ -435,7 +397,7 @@ leaderElection:
 			flags: []string{
 				"--leader-elect=false",
 				"--leader-elect-lease-duration=2h", // CLI args are favored over the fields in ComponentConfig
-				"--lock-object-namespace=default",  // deprecated CLI arg will be ignored if --config is specified
+				"--kubeconfig=foo",                 // deprecated CLI arg will be ignored if --config is specified
 				"--config", emptyLeaderElectionConfig,
 			},
 			wantLeaderElection: &componentbaseconfig.LeaderElectionConfiguration{
@@ -447,14 +409,20 @@ leaderElection:
 				ResourceName:      v1beta3.SchedulerDefaultLockObjectName,
 				ResourceNamespace: v1beta3.SchedulerDefaultLockObjectNamespace,
 			},
+			wantClientConnection: &componentbaseconfig.ClientConnectionConfiguration{
+				Kubeconfig:  configKubeconfig,
+				ContentType: "application/vnd.kubernetes.protobuf",
+				QPS:         50,
+				Burst:       100,
+			},
 		},
 		{
 			name: "leader election CLI args, without --config arg",
 			flags: []string{
 				"--leader-elect=false",
 				"--leader-elect-lease-duration=2h",
-				"--lock-object-namespace=default", // deprecated CLI arg is honored if --config is not specified
-				"--kubeconfig", configKubeconfig,
+				"--leader-elect-resource-namespace=default",
+				"--kubeconfig", configKubeconfig, // deprecated CLI arg is honored if --config is not specified
 			},
 			wantLeaderElection: &componentbaseconfig.LeaderElectionConfiguration{
 				LeaderElect:       false,                                    // from CLI args
@@ -463,7 +431,13 @@ leaderElection:
 				RetryPeriod:       metav1.Duration{Duration: 2 * time.Second},
 				ResourceLock:      "leases",
 				ResourceName:      v1beta3.SchedulerDefaultLockObjectName,
-				ResourceNamespace: "default", // from deprecated CLI args
+				ResourceNamespace: "default", // from CLI args
+			},
+			wantClientConnection: &componentbaseconfig.ClientConnectionConfiguration{
+				Kubeconfig:  configKubeconfig, // from deprecated CLI args
+				ContentType: "application/vnd.kubernetes.protobuf",
+				QPS:         50,
+				Burst:       100,
 			},
 		},
 		{
@@ -559,6 +533,13 @@ leaderElection:
 					t.Errorf("Unexpected leaderElection diff (-want, +got): %s", diff)
 				}
 			}
+
+			if tc.wantClientConnection != nil {
+				gotClientConnection := opts.ComponentConfig.ClientConnection
+				if diff := cmp.Diff(*tc.wantClientConnection, gotClientConnection); diff != "" {
+					t.Errorf("Unexpected clientConnection diff (-want, +got): %s", diff)
+				}
+			}
 		})
 	}
 }
@@ -577,8 +558,8 @@ func newFoo(_ runtime.Object, _ framework.Handle) (framework.Plugin, error) {
 	return &foo{}, nil
 }
 
-func (*foo) PreFilter(_ context.Context, _ *framework.CycleState, _ *v1.Pod) *framework.Status {
-	return nil
+func (*foo) PreFilter(_ context.Context, _ *framework.CycleState, _ *v1.Pod) (*framework.PreFilterResult, *framework.Status) {
+	return nil, nil
 }
 
 func (*foo) PreFilterExtensions() framework.PreFilterExtensions {
