@@ -147,7 +147,7 @@ kube::version::load_version_vars() {
 # Prints the value that needs to be passed to the -ldflags parameter of go build
 # in order to set the Kubernetes based on the git tree status.
 # IMPORTANT: if you update any of these, also update the lists in
-# pkg/version/def.bzl and hack/print-workspace-status.sh.
+# hack/print-workspace-status.sh.
 kube::version::ldflags() {
   kube::version::get_version_vars
 
@@ -155,7 +155,6 @@ kube::version::ldflags() {
   function add_ldflag() {
     local key=${1}
     local val=${2}
-    # If you update these, also update the list component-base/version/def.bzl.
     ldflags+=(
       "-X '${KUBE_GO_PACKAGE}/vendor/k8s.io/client-go/pkg/version.${key}=${val}'"
       "-X '${KUBE_GO_PACKAGE}/vendor/k8s.io/component-base/version.${key}=${val}'"
@@ -164,7 +163,9 @@ kube::version::ldflags() {
     )
   }
 
-  add_ldflag "buildDate" "$(date ${SOURCE_DATE_EPOCH:+"--date=@${SOURCE_DATE_EPOCH}"} -u +'%Y-%m-%dT%H:%M:%SZ')"
+  kube::util::ensure-gnu-date
+
+  add_ldflag "buildDate" "$(${DATE} ${SOURCE_DATE_EPOCH:+"--date=@${SOURCE_DATE_EPOCH}"} -u +'%Y-%m-%dT%H:%M:%SZ')"
   if [[ -n ${KUBE_GIT_COMMIT-} ]]; then
     add_ldflag "gitCommit" "${KUBE_GIT_COMMIT}"
     add_ldflag "gitTreeState" "${KUBE_GIT_TREE_STATE}"
