@@ -28,6 +28,9 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
+	utilfeature "k8s.io/apiserver/pkg/util/feature"
+	featuregatetesting "k8s.io/component-base/featuregate/testing"
+	"k8s.io/kubernetes/pkg/features"
 )
 
 func TestNewResource(t *testing.T) {
@@ -1336,26 +1339,26 @@ func TestGetNamespacesFromPodAffinityTerm(t *testing.T) {
 	tests := []struct {
 		name string
 		term *v1.PodAffinityTerm
-		want sets.String
+		want sets.Set[string]
 	}{
 		{
 			name: "podAffinityTerm_namespace_empty",
 			term: &v1.PodAffinityTerm{},
-			want: sets.String{metav1.NamespaceDefault: sets.Empty{}},
+			want: sets.Set[string]{metav1.NamespaceDefault: sets.Empty{}},
 		},
 		{
 			name: "podAffinityTerm_namespace_not_empty",
 			term: &v1.PodAffinityTerm{
 				Namespaces: []string{metav1.NamespacePublic, metav1.NamespaceSystem},
 			},
-			want: sets.NewString(metav1.NamespacePublic, metav1.NamespaceSystem),
+			want: sets.New(metav1.NamespacePublic, metav1.NamespaceSystem),
 		},
 		{
 			name: "podAffinityTerm_namespace_selector_not_nil",
 			term: &v1.PodAffinityTerm{
 				NamespaceSelector: &metav1.LabelSelector{},
 			},
-			want: sets.String{},
+			want: sets.Set[string]{},
 		},
 	}
 
@@ -1373,3 +1376,234 @@ func TestGetNamespacesFromPodAffinityTerm(t *testing.T) {
 		})
 	}
 }
+
+func TestFitError_Error(t *testing.T) {
+	tests := []struct {
+		name          string
+		pod           *v1.Pod
+		numAllNodes   int
+		diagnosis     Diagnosis
+		wantReasonMsg string
+	}{
+		{
+			name:        "nodes failed Prefilter plugin",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "Node(s) failed PreFilter plugin FalsePreFilter",
+				NodeToStatusMap: NodeToStatusMap{
+					// They're inserted by the framework.
+					// We don't include them in the reason message because they'd be just duplicates.
+					"node1": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+				},
+			},
+			wantReasonMsg: "0/3 nodes are available: Node(s) failed PreFilter plugin FalsePreFilter.",
+		},
+		{
+			name:        "nodes failed Prefilter plugin and the preemption also failed",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "Node(s) failed PreFilter plugin FalsePreFilter",
+				NodeToStatusMap: NodeToStatusMap{
+					// They're inserted by the framework.
+					// We don't include them in the reason message because they'd be just duplicates.
+					"node1": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed PreFilter plugin FalsePreFilter"),
+				},
+				// PostFilterMsg will be included.
+				PostFilterMsg: "Error running PostFilter plugin FailedPostFilter",
+			},
+			wantReasonMsg: "0/3 nodes are available: Node(s) failed PreFilter plugin FalsePreFilter. Error running PostFilter plugin FailedPostFilter.",
+		},
+		{
+			name:        "nodes failed one Filter plugin with an empty PostFilterMsg",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "",
+				NodeToStatusMap: NodeToStatusMap{
+					"node1": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+				},
+			},
+			wantReasonMsg: "0/3 nodes are available: 3 Node(s) failed Filter plugin FalseFilter-1.",
+		},
+		{
+			name:        "nodes failed one Filter plugin with a non-empty PostFilterMsg",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "",
+				NodeToStatusMap: NodeToStatusMap{
+					"node1": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+				},
+				PostFilterMsg: "Error running PostFilter plugin FailedPostFilter",
+			},
+			wantReasonMsg: "0/3 nodes are available: 3 Node(s) failed Filter plugin FalseFilter-1. Error running PostFilter plugin FailedPostFilter.",
+		},
+		{
+			name:        "nodes failed two Filter plugins with an empty PostFilterMsg",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "",
+				NodeToStatusMap: NodeToStatusMap{
+					"node1": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-2"),
+				},
+			},
+			wantReasonMsg: "0/3 nodes are available: 1 Node(s) failed Filter plugin FalseFilter-2, 2 Node(s) failed Filter plugin FalseFilter-1.",
+		},
+		{
+			name:        "nodes failed two Filter plugins with a non-empty PostFilterMsg",
+			numAllNodes: 3,
+			diagnosis: Diagnosis{
+				PreFilterMsg: "",
+				NodeToStatusMap: NodeToStatusMap{
+					"node1": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node2": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-1"),
+					"node3": NewStatus(Unschedulable, "Node(s) failed Filter plugin FalseFilter-2"),
+				},
+				PostFilterMsg: "Error running PostFilter plugin FailedPostFilter",
+			},
+			wantReasonMsg: "0/3 nodes are available: 1 Node(s) failed Filter plugin FalseFilter-2, 2 Node(s) failed Filter plugin FalseFilter-1. Error running PostFilter plugin FailedPostFilter.",
+		},
+		{
+			name:        "failed to Permit on node",
+			numAllNodes: 1,
+			diagnosis: Diagnosis{
+				NodeToStatusMap: NodeToStatusMap{
+					// There should be only one node here.
+					"node1": NewStatus(Unschedulable, "Node failed Permit plugin Permit-1"),
+				},
+			},
+			wantReasonMsg: "0/1 nodes are available: 1 Node failed Permit plugin Permit-1.",
+		},
+		{
+			name:        "failed to Reserve on node",
+			numAllNodes: 1,
+			diagnosis: Diagnosis{
+				NodeToStatusMap: NodeToStatusMap{
+					// There should be only one node here.
+					"node1": NewStatus(Unschedulable, "Node failed Reserve plugin Reserve-1"),
+				},
+			},
+			wantReasonMsg: "0/1 nodes are available: 1 Node failed Reserve plugin Reserve-1.",
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			f := &FitError{
+				Pod:         tt.pod,
+				NumAllNodes: tt.numAllNodes,
+				Diagnosis:   tt.diagnosis,
+			}
+			if gotReasonMsg := f.Error(); gotReasonMsg != tt.wantReasonMsg {
+				t.Errorf("Error() = Got: %v Want: %v", gotReasonMsg, tt.wantReasonMsg)
+			}
+		})
+	}
+}
+
+func TestCalculatePodResourcesWithResize(t *testing.T) {
+	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.InPlacePodVerticalScaling, true)()
+	cpu500m := resource.MustParse("500m")
+	mem500M := resource.MustParse("500Mi")
+	cpu700m := resource.MustParse("700m")
+	mem800M := resource.MustParse("800Mi")
+	testpod := v1.Pod{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: "pod_resize_test",
+			Name:      "testpod",
+			UID:       types.UID("testpod"),
+		},
+		Spec: v1.PodSpec{
+			Containers: []v1.Container{
+				{
+					Name:      "c1",
+					Resources: v1.ResourceRequirements{Requests: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M}},
+				},
+			},
+		},
+		Status: v1.PodStatus{
+			Phase:  v1.PodRunning,
+			Resize: "",
+			ContainerStatuses: []v1.ContainerStatus{
+				{
+					Name:               "c1",
+					AllocatedResources: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+				},
+			},
+		},
+	}
+
+	tests := []struct {
+		name               string
+		requests           v1.ResourceList
+		allocatedResources v1.ResourceList
+		resizeStatus       v1.PodResizeStatus
+		expectedResource   Resource
+		expectedNon0CPU    int64
+		expectedNon0Mem    int64
+	}{
+		{
+			name:               "Pod with no pending resize",
+			requests:           v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			allocatedResources: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			resizeStatus:       "",
+			expectedResource:   Resource{MilliCPU: cpu500m.MilliValue(), Memory: mem500M.Value()},
+			expectedNon0CPU:    cpu500m.MilliValue(),
+			expectedNon0Mem:    mem500M.Value(),
+		},
+		{
+			name:               "Pod with resize in progress",
+			requests:           v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			allocatedResources: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			resizeStatus:       v1.PodResizeStatusInProgress,
+			expectedResource:   Resource{MilliCPU: cpu500m.MilliValue(), Memory: mem500M.Value()},
+			expectedNon0CPU:    cpu500m.MilliValue(),
+			expectedNon0Mem:    mem500M.Value(),
+		},
+		{
+			name:               "Pod with deferred resize",
+			requests:           v1.ResourceList{v1.ResourceCPU: cpu700m, v1.ResourceMemory: mem800M},
+			allocatedResources: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			resizeStatus:       v1.PodResizeStatusDeferred,
+			expectedResource:   Resource{MilliCPU: cpu700m.MilliValue(), Memory: mem800M.Value()},
+			expectedNon0CPU:    cpu700m.MilliValue(),
+			expectedNon0Mem:    mem800M.Value(),
+		},
+		{
+			name:               "Pod with infeasible resize",
+			requests:           v1.ResourceList{v1.ResourceCPU: cpu700m, v1.ResourceMemory: mem800M},
+			allocatedResources: v1.ResourceList{v1.ResourceCPU: cpu500m, v1.ResourceMemory: mem500M},
+			resizeStatus:       v1.PodResizeStatusInfeasible,
+			expectedResource:   Resource{MilliCPU: cpu500m.MilliValue(), Memory: mem500M.Value()},
+			expectedNon0CPU:    cpu500m.MilliValue(),
+			expectedNon0Mem:    mem500M.Value(),
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			pod := testpod.DeepCopy()
+			pod.Spec.Containers[0].Resources.Requests = tt.requests
+			pod.Status.ContainerStatuses[0].AllocatedResources = tt.allocatedResources
+			pod.Status.Resize = tt.resizeStatus
+
+			res, non0CPU, non0Mem := calculateResource(pod)
+			if !reflect.DeepEqual(tt.expectedResource, res) {
+				t.Errorf("Test: %s expected resource: %+v, got: %+v", tt.name, tt.expectedResource, res)
+			}
+			if non0CPU != tt.expectedNon0CPU {
+				t.Errorf("Test: %s expected non0CPU: %d, got: %d", tt.name, tt.expectedNon0CPU, non0CPU)
+			}
+			if non0Mem != tt.expectedNon0Mem {
+				t.Errorf("Test: %s expected non0Mem: %d, got: %d", tt.name, tt.expectedNon0Mem, non0Mem)
+			}
+		})
+	}
+}
