@@ -18,8 +18,8 @@ package delete
 
 import (
 	"encoding/json"
+	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"strconv"
 	"strings"
@@ -30,12 +30,13 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/cli-runtime/pkg/genericclioptions"
+	"k8s.io/cli-runtime/pkg/genericiooptions"
 	"k8s.io/cli-runtime/pkg/resource"
 	"k8s.io/client-go/rest/fake"
 	cmdtesting "k8s.io/kubectl/pkg/cmd/testing"
 	cmdutil "k8s.io/kubectl/pkg/cmd/util"
 	"k8s.io/kubectl/pkg/scheme"
+	"k8s.io/utils/pointer"
 )
 
 func fakecmd() *cobra.Command {
@@ -48,6 +49,49 @@ func fakecmd() *cobra.Command {
 	return cmd
 }
 
+func TestDeleteFlagValidation(t *testing.T) {
+	f := cmdtesting.NewTestFactory()
+	defer f.Cleanup()
+
+	tests := []struct {
+		flags        DeleteFlags
+		enableAlphas []cmdutil.FeatureGate
+		args         [][]string
+		expectedErr  string
+	}{
+		{
+			flags: DeleteFlags{
+				Raw:         pointer.String("test"),
+				Interactive: pointer.Bool(true),
+			},
+			enableAlphas: []cmdutil.FeatureGate{cmdutil.InteractiveDelete},
+			expectedErr:  "--interactive can not be used with --raw",
+		},
+	}
+
+	for _, test := range tests {
+		cmd := fakecmd()
+		cmdtesting.WithAlphaEnvs(test.enableAlphas, t, func(t *testing.T) {
+			deleteOptions, err := test.flags.ToOptions(nil, genericiooptions.NewTestIOStreamsDiscard())
+			if err != nil {
+				t.Fatalf("unexpected error creating delete options: %s", err)
+			}
+			deleteOptions.Filenames = []string{"../../../testdata/redis-master-controller.yaml"}
+			err = deleteOptions.Complete(f, nil, cmd)
+			if err != nil {
+				t.Fatalf("unexpected error creating delete options: %s", err)
+			}
+			err = deleteOptions.Validate()
+			if err == nil {
+				t.Fatalf("missing expected error")
+			}
+			if test.expectedErr != err.Error() {
+				t.Errorf("expected error %s, got %s", test.expectedErr, err)
+			}
+		})
+	}
+}
+
 func TestDeleteObjectByTuple(t *testing.T) {
 	cmdtesting.InitTestErrorHandler(t)
 	_, _, rc := cmdtesting.TestData()
@@ -78,7 +122,7 @@ func TestDeleteObjectByTuple(t *testing.T) {
 		}),
 	}
 
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("cascade", "false")
@@ -89,7 +133,7 @@ func TestDeleteObjectByTuple(t *testing.T) {
 	}
 
 	// Test cascading delete of object without client-side reaper doesn't make GET requests
-	streams, _, buf, _ = genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ = genericiooptions.NewTestIOStreams()
 	cmd = NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("output", "name")
@@ -104,7 +148,7 @@ func hasExpectedPropagationPolicy(body io.ReadCloser, policy *metav1.DeletionPro
 		return body == nil && policy == nil
 	}
 	var parsedBody metav1.DeleteOptions
-	rawBody, _ := ioutil.ReadAll(body)
+	rawBody, _ := io.ReadAll(body)
 	json.Unmarshal(rawBody, &parsedBody)
 	if parsedBody.PropagationPolicy == nil {
 		return false
@@ -140,7 +184,7 @@ func TestCascadingStrategy(t *testing.T) {
 	// DeleteOptions.PropagationPolicy should be Background, when cascading strategy is empty (default).
 	backgroundPolicy := metav1.DeletePropagationBackground
 	policy = &backgroundPolicy
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("output", "name")
@@ -152,7 +196,7 @@ func TestCascadingStrategy(t *testing.T) {
 	// DeleteOptions.PropagationPolicy should be Foreground, when cascading strategy is foreground.
 	foregroundPolicy := metav1.DeletePropagationForeground
 	policy = &foregroundPolicy
-	streams, _, buf, _ = genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ = genericiooptions.NewTestIOStreams()
 	cmd = NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("cascade", "foreground")
@@ -165,7 +209,7 @@ func TestCascadingStrategy(t *testing.T) {
 	// Test that delete options should be set to orphan when cascading strategy is orphan.
 	orphanPolicy := metav1.DeletePropagationOrphan
 	policy = &orphanPolicy
-	streams, _, buf, _ = genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ = genericiooptions.NewTestIOStreams()
 	cmd = NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("cascade", "orphan")
@@ -207,7 +251,7 @@ func TestDeleteNamedObject(t *testing.T) {
 		}),
 	}
 
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("cascade", "false")
@@ -218,7 +262,7 @@ func TestDeleteNamedObject(t *testing.T) {
 	}
 
 	// Test cascading delete of object without client-side reaper doesn't make GET requests
-	streams, _, buf, _ = genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ = genericiooptions.NewTestIOStreams()
 	cmd = NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
 	cmd.Flags().Set("cascade", "false")
@@ -251,7 +295,7 @@ func TestDeleteObject(t *testing.T) {
 		}),
 	}
 
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("filename", "../../../testdata/redis-master-controller.yaml")
 	cmd.Flags().Set("cascade", "false")
@@ -264,6 +308,90 @@ func TestDeleteObject(t *testing.T) {
 	}
 }
 
+func TestPreviewResultEqualToResult(t *testing.T) {
+	deleteFlags := NewDeleteCommandFlags("")
+	deleteFlags.Interactive = pointer.Bool(true)
+
+	tf := cmdtesting.NewTestFactory().WithNamespace("test")
+	defer tf.Cleanup()
+
+	streams, _, _, _ := genericiooptions.NewTestIOStreams()
+
+	deleteOptions, err := deleteFlags.ToOptions(nil, streams)
+	deleteOptions.Filenames = []string{"../../../testdata/redis-master-controller.yaml"}
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+	err = deleteOptions.Complete(tf, nil, fakecmd())
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	infos, err := deleteOptions.Result.Infos()
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+	previewInfos, err := deleteOptions.PreviewResult.Infos()
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+	if len(infos) != len(previewInfos) {
+		t.Errorf("result and previewResult must match")
+	}
+}
+
+func TestDeleteObjectWithInteractive(t *testing.T) {
+	cmdtesting.InitTestErrorHandler(t)
+	_, _, rc := cmdtesting.TestData()
+
+	tf := cmdtesting.NewTestFactory().WithNamespace("test")
+	defer tf.Cleanup()
+
+	codec := scheme.Codecs.LegacyCodec(scheme.Scheme.PrioritizedVersionsAllGroups()...)
+
+	tf.UnstructuredClient = &fake.RESTClient{
+		NegotiatedSerializer: resource.UnstructuredPlusDefaultContentConfig().NegotiatedSerializer,
+		Client: fake.CreateHTTPClient(func(req *http.Request) (*http.Response, error) {
+			switch p, m := req.URL.Path, req.Method; {
+			case p == "/namespaces/test/replicationcontrollers/redis-master" && m == "DELETE":
+				return &http.Response{StatusCode: http.StatusOK, Header: cmdtesting.DefaultHeader(), Body: cmdtesting.ObjBody(codec, &rc.Items[0])}, nil
+			default:
+				t.Fatalf("unexpected request: %#v\n%#v", req.URL, req)
+				return nil, nil
+			}
+		}),
+	}
+
+	cmdtesting.WithAlphaEnvs([]cmdutil.FeatureGate{cmdutil.InteractiveDelete}, t, func(t *testing.T) {
+		streams, in, buf, _ := genericiooptions.NewTestIOStreams()
+		fmt.Fprint(in, "y")
+		cmd := NewCmdDelete(tf, streams)
+		cmd.Flags().Set("filename", "../../../testdata/redis-master-controller.yaml")
+		cmd.Flags().Set("output", "name")
+		cmd.Flags().Set("interactive", "true")
+		cmd.Run(cmd, []string{})
+
+		if buf.String() != "You are about to delete the following 1 resource(s):\nreplicationcontroller/redis-master\nDo you want to continue? (y/n): replicationcontroller/redis-master\n" {
+			t.Errorf("unexpected output: %s", buf.String())
+		}
+
+		streams, in, buf, _ = genericiooptions.NewTestIOStreams()
+		fmt.Fprint(in, "n")
+		cmd = NewCmdDelete(tf, streams)
+		cmd.Flags().Set("filename", "../../../testdata/redis-master-controller.yaml")
+		cmd.Flags().Set("output", "name")
+		cmd.Flags().Set("interactive", "true")
+		cmd.Run(cmd, []string{})
+
+		if buf.String() != "You are about to delete the following 1 resource(s):\nreplicationcontroller/redis-master\nDo you want to continue? (y/n): deletion is cancelled\n" {
+			t.Errorf("unexpected output: %s", buf.String())
+		}
+		if buf.String() == ": replicationcontroller/redis-master\n" {
+			t.Errorf("unexpected output: %s", buf.String())
+		}
+	})
+}
+
 func TestGracePeriodScenarios(t *testing.T) {
 	pods, _, _ := cmdtesting.TestData()
 
@@ -290,7 +418,7 @@ func TestGracePeriodScenarios(t *testing.T) {
 			forceFlag:                 true,
 			expectedGracePeriod:       "0",
 			expectedOut:               "pod/foo\n",
-			expectedErrOut:            "warning: Immediate deletion does not wait for confirmation that the running resource has been terminated. The resource may continue to run on the cluster indefinitely.\n",
+			expectedErrOut:            "Warning: Immediate deletion does not wait for confirmation that the running resource has been terminated. The resource may continue to run on the cluster indefinitely.\n",
 			expectedDeleteRequestPath: "/namespaces/test/pods/foo",
 		},
 		{
@@ -300,7 +428,7 @@ func TestGracePeriodScenarios(t *testing.T) {
 			gracePeriodFlag:           "0",
 			expectedGracePeriod:       "0",
 			expectedOut:               "pod/foo\n",
-			expectedErrOut:            "warning: Immediate deletion does not wait for confirmation that the running resource has been terminated. The resource may continue to run on the cluster indefinitely.\n",
+			expectedErrOut:            "Warning: Immediate deletion does not wait for confirmation that the running resource has been terminated. The resource may continue to run on the cluster indefinitely.\n",
 			expectedDeleteRequestPath: "/namespaces/test/pods/foo",
 		},
 		{
@@ -387,7 +515,7 @@ func TestGracePeriodScenarios(t *testing.T) {
 			}
 
 			// Test the command using the flags specified in the test case
-			streams, _, out, errOut := genericclioptions.NewTestIOStreams()
+			streams, _, out, errOut := genericiooptions.NewTestIOStreams()
 			cmd := NewCmdDelete(tf, streams)
 			cmd.Flags().Set("output", "name")
 			if test.forceFlag {
@@ -443,7 +571,7 @@ func TestDeleteObjectNotFound(t *testing.T) {
 		GracePeriod:       -1,
 		CascadingStrategy: metav1.DeletePropagationOrphan,
 		Output:            "name",
-		IOStreams:         genericclioptions.NewTestIOStreamsDiscard(),
+		IOStreams:         genericiooptions.NewTestIOStreamsDiscard(),
 	}
 	err := options.Complete(tf, []string{}, fakecmd())
 	if err != nil {
@@ -472,7 +600,7 @@ func TestDeleteObjectIgnoreNotFound(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("filename", "../../../testdata/redis-master-controller.yaml")
@@ -523,7 +651,7 @@ func TestDeleteAllNotFound(t *testing.T) {
 		DeleteAll:         true,
 		IgnoreNotFound:    false,
 		Output:            "name",
-		IOStreams:         genericclioptions.NewTestIOStreamsDiscard(),
+		IOStreams:         genericiooptions.NewTestIOStreamsDiscard(),
 	}
 	err := options.Complete(tf, []string{"services"}, fakecmd())
 	if err != nil {
@@ -564,7 +692,7 @@ func TestDeleteAllIgnoreNotFound(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("all", "true")
@@ -600,7 +728,7 @@ func TestDeleteMultipleObject(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("filename", "../../../testdata/redis-master-controller.yaml")
@@ -637,7 +765,7 @@ func TestDeleteMultipleObjectContinueOnMissing(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	options := &DeleteOptions{
 		FilenameOptions: resource.FilenameOptions{
@@ -689,7 +817,7 @@ func TestDeleteMultipleResourcesWithTheSameName(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("namespace", "test")
@@ -722,7 +850,7 @@ func TestDeleteDirectory(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("filename", "../../../testdata/replace/legacy")
@@ -768,7 +896,7 @@ func TestDeleteMultipleSelector(t *testing.T) {
 			}
 		}),
 	}
-	streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+	streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 
 	cmd := NewCmdDelete(tf, streams)
 	cmd.Flags().Set("selector", "a=b")
@@ -812,7 +940,7 @@ func TestResourceErrors(t *testing.T) {
 
 			tf.ClientConfigVal = cmdtesting.DefaultClientConfig()
 
-			streams, _, buf, _ := genericclioptions.NewTestIOStreams()
+			streams, _, buf, _ := genericiooptions.NewTestIOStreams()
 			options := &DeleteOptions{
 				FilenameOptions:   resource.FilenameOptions{},
 				GracePeriod:       -1,
