@@ -19,12 +19,13 @@ package apps
 import (
 	"context"
 	"fmt"
-	"github.com/onsi/gomega"
 	"strings"
 	"time"
 
+	"github.com/onsi/gomega"
+
 	jsonpatch "github.com/evanphx/json-patch"
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
@@ -63,7 +64,7 @@ var defaultLabels = map[string]string{"foo": "bar"}
 
 var _ = SIGDescribe("DisruptionController", func() {
 	f := framework.NewDefaultFramework("disruption")
-	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
+	f.NamespacePodSecurityLevel = admissionapi.LevelPrivileged
 	var ns string
 	var cs kubernetes.Interface
 	var dc dynamic.Interface
@@ -76,26 +77,26 @@ var _ = SIGDescribe("DisruptionController", func() {
 
 	ginkgo.Context("Listing PodDisruptionBudgets for all namespaces", func() {
 		anotherFramework := framework.NewDefaultFramework("disruption-2")
-		anotherFramework.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
+		anotherFramework.NamespacePodSecurityLevel = admissionapi.LevelPrivileged
 
 		/*
 		   Release : v1.21
 		   Testname: PodDisruptionBudget: list and delete collection
 		   Description: PodDisruptionBudget API must support list and deletecollection operations.
 		*/
-		framework.ConformanceIt("should list and delete a collection of PodDisruptionBudgets", func() {
+		framework.ConformanceIt("should list and delete a collection of PodDisruptionBudgets", func(ctx context.Context) {
 			specialLabels := map[string]string{"foo_pdb": "bar_pdb"}
 			labelSelector := labels.SelectorFromSet(specialLabels).String()
-			createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromInt(2), specialLabels)
-			createPDBMinAvailableOrDie(cs, ns, "foo2", intstr.FromString("1%"), specialLabels)
-			createPDBMinAvailableOrDie(anotherFramework.ClientSet, anotherFramework.Namespace.Name, "foo3", intstr.FromInt(2), specialLabels)
+			createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromInt(2), specialLabels)
+			createPDBMinAvailableOrDie(ctx, cs, ns, "foo2", intstr.FromString("1%"), specialLabels)
+			createPDBMinAvailableOrDie(ctx, anotherFramework.ClientSet, anotherFramework.Namespace.Name, "foo3", intstr.FromInt(2), specialLabels)
 
 			ginkgo.By("listing a collection of PDBs across all namespaces")
-			listPDBs(cs, metav1.NamespaceAll, labelSelector, 3, []string{defaultName, "foo2", "foo3"})
+			listPDBs(ctx, cs, metav1.NamespaceAll, labelSelector, 3, []string{defaultName, "foo2", "foo3"})
 
 			ginkgo.By("listing a collection of PDBs in namespace " + ns)
-			listPDBs(cs, ns, labelSelector, 2, []string{defaultName, "foo2"})
-			deletePDBCollection(cs, ns)
+			listPDBs(ctx, cs, ns, labelSelector, 2, []string{defaultName, "foo2"})
+			deletePDBCollection(ctx, cs, ns)
 		})
 	})
 
@@ -104,12 +105,12 @@ var _ = SIGDescribe("DisruptionController", func() {
 		Testname: PodDisruptionBudget: create, update, patch, and delete object
 		Description: PodDisruptionBudget API must support create, update, patch, and delete operations.
 	*/
-	framework.ConformanceIt("should create a PodDisruptionBudget", func() {
+	framework.ConformanceIt("should create a PodDisruptionBudget", func(ctx context.Context) {
 		ginkgo.By("creating the pdb")
-		createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromString("1%"), defaultLabels)
+		createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromString("1%"), defaultLabels)
 
 		ginkgo.By("updating the pdb")
-		updatedPDB := updatePDBOrDie(cs, ns, defaultName, func(pdb *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
+		updatedPDB := updatePDBOrDie(ctx, cs, ns, defaultName, func(pdb *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
 			newMinAvailable := intstr.FromString("2%")
 			pdb.Spec.MinAvailable = &newMinAvailable
 			return pdb
@@ -117,7 +118,7 @@ var _ = SIGDescribe("DisruptionController", func() {
 		framework.ExpectEqual(updatedPDB.Spec.MinAvailable.String(), "2%")
 
 		ginkgo.By("patching the pdb")
-		patchedPDB := patchPDBOrDie(cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
+		patchedPDB := patchPDBOrDie(ctx, cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
 			newBytes, err := json.Marshal(map[string]interface{}{
 				"spec": map[string]interface{}{
 					"minAvailable": "3%",
@@ -128,7 +129,7 @@ var _ = SIGDescribe("DisruptionController", func() {
 		})
 		framework.ExpectEqual(patchedPDB.Spec.MinAvailable.String(), "3%")
 
-		deletePDBOrDie(cs, ns, defaultName)
+		deletePDBOrDie(ctx, cs, ns, defaultName)
 	})
 
 	/*
@@ -137,16 +138,16 @@ var _ = SIGDescribe("DisruptionController", func() {
 	   Description: Disruption controller MUST update the PDB status with
 	   how many disruptions are allowed.
 	*/
-	framework.ConformanceIt("should observe PodDisruptionBudget status updated", func() {
-		createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
+	framework.ConformanceIt("should observe PodDisruptionBudget status updated", func(ctx context.Context) {
+		createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
 
-		createPodsOrDie(cs, ns, 3)
-		waitForPodsOrDie(cs, ns, 3)
+		createPodsOrDie(ctx, cs, ns, 3)
+		waitForPodsOrDie(ctx, cs, ns, 3)
 
 		// Since disruptionAllowed starts out 0, if we see it ever become positive,
 		// that means the controller is working.
-		err := wait.PollImmediate(framework.Poll, timeout, func() (bool, error) {
-			pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), defaultName, metav1.GetOptions{})
+		err := wait.PollImmediateWithContext(ctx, framework.Poll, timeout, func(ctx context.Context) (bool, error) {
+			pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, defaultName, metav1.GetOptions{})
 			if err != nil {
 				return false, err
 			}
@@ -160,26 +161,26 @@ var _ = SIGDescribe("DisruptionController", func() {
 		Testname: PodDisruptionBudget: update and patch status
 		Description: PodDisruptionBudget API must support update and patch operations on status subresource.
 	*/
-	framework.ConformanceIt("should update/patch PodDisruptionBudget status", func() {
-		createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
+	framework.ConformanceIt("should update/patch PodDisruptionBudget status", func(ctx context.Context) {
+		createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
 
 		ginkgo.By("Updating PodDisruptionBudget status")
 		// PDB status can be updated by both PDB controller and the status API. The test selects `DisruptedPods` field to show immediate update via API.
 		// The pod has to exist, otherwise wil be removed by the controller. Other fields may not reflect the change from API.
-		createPodsOrDie(cs, ns, 1)
-		waitForPodsOrDie(cs, ns, 1)
-		pod, _ := locateRunningPod(cs, ns)
-		updatePDBOrDie(cs, ns, defaultName, func(old *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
+		createPodsOrDie(ctx, cs, ns, 1)
+		waitForPodsOrDie(ctx, cs, ns, 1)
+		pod, _ := locateRunningPod(ctx, cs, ns)
+		updatePDBOrDie(ctx, cs, ns, defaultName, func(old *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
 			old.Status.DisruptedPods = make(map[string]metav1.Time)
 			old.Status.DisruptedPods[pod.Name] = metav1.NewTime(time.Now())
 			return old
 		}, cs.PolicyV1().PodDisruptionBudgets(ns).UpdateStatus)
 		// fetch again to make sure the update from API was effective
-		updated := getPDBStatusOrDie(dc, ns, defaultName)
+		updated := getPDBStatusOrDie(ctx, dc, ns, defaultName)
 		framework.ExpectHaveKey(updated.Status.DisruptedPods, pod.Name, "Expecting the DisruptedPods have %s", pod.Name)
 
 		ginkgo.By("Patching PodDisruptionBudget status")
-		patched := patchPDBOrDie(cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
+		patched := patchPDBOrDie(ctx, cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
 			oldBytes, err := json.Marshal(old)
 			framework.ExpectNoError(err, "failed to marshal JSON for old data")
 			old.Status.DisruptedPods = make(map[string]metav1.Time)
@@ -192,15 +193,15 @@ var _ = SIGDescribe("DisruptionController", func() {
 
 	// PDB shouldn't error out when there are unmanaged pods
 	ginkgo.It("should observe that the PodDisruptionBudget status is not updated for unmanaged pods",
-		func() {
-			createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
+		func(ctx context.Context) {
+			createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromInt(1), defaultLabels)
 
-			createPodsOrDie(cs, ns, 3)
-			waitForPodsOrDie(cs, ns, 3)
+			createPodsOrDie(ctx, cs, ns, 3)
+			waitForPodsOrDie(ctx, cs, ns, 3)
 
 			// Since we allow unmanaged pods to be associated with a PDB, we should not see any error
-			gomega.Consistently(func() (bool, error) {
-				pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), defaultName, metav1.GetOptions{})
+			gomega.Consistently(ctx, func(ctx context.Context) (bool, error) {
+				pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, defaultName, metav1.GetOptions{})
 				if err != nil {
 					return false, err
 				}
@@ -286,25 +287,25 @@ var _ = SIGDescribe("DisruptionController", func() {
 		if c.exclusive {
 			serial = " [Serial]"
 		}
-		ginkgo.It(fmt.Sprintf("evictions: %s => %s%s", c.description, expectation, serial), func() {
+		ginkgo.It(fmt.Sprintf("evictions: %s => %s%s", c.description, expectation, serial), func(ctx context.Context) {
 			if c.skipForBigClusters {
 				e2eskipper.SkipUnlessNodeCountIsAtMost(bigClusterSize - 1)
 			}
-			createPodsOrDie(cs, ns, c.podCount)
+			createPodsOrDie(ctx, cs, ns, c.podCount)
 			if c.replicaSetSize > 0 {
-				createReplicaSetOrDie(cs, ns, c.replicaSetSize, c.exclusive)
+				createReplicaSetOrDie(ctx, cs, ns, c.replicaSetSize, c.exclusive)
 			}
 
 			if c.minAvailable.String() != "" {
-				createPDBMinAvailableOrDie(cs, ns, defaultName, c.minAvailable, defaultLabels)
+				createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, c.minAvailable, defaultLabels)
 			}
 
 			if c.maxUnavailable.String() != "" {
-				createPDBMaxUnavailableOrDie(cs, ns, defaultName, c.maxUnavailable)
+				createPDBMaxUnavailableOrDie(ctx, cs, ns, defaultName, c.maxUnavailable)
 			}
 
 			// Locate a running pod.
-			pod, err := locateRunningPod(cs, ns)
+			pod, err := locateRunningPod(ctx, cs, ns)
 			framework.ExpectNoError(err)
 
 			e := &policyv1.Eviction{
@@ -315,19 +316,21 @@ var _ = SIGDescribe("DisruptionController", func() {
 			}
 
 			if c.shouldDeny {
-				err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+				err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 				framework.ExpectError(err, "pod eviction should fail")
-				framework.ExpectEqual(apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause), true, "pod eviction should fail with DisruptionBudget cause")
+				if !apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause) {
+					framework.Fail("pod eviction should fail with DisruptionBudget cause")
+				}
 			} else {
 				// Only wait for running pods in the "allow" case
 				// because one of shouldDeny cases relies on the
 				// replicaSet not fitting on the cluster.
-				waitForPodsOrDie(cs, ns, c.podCount+int(c.replicaSetSize))
+				waitForPodsOrDie(ctx, cs, ns, c.podCount+int(c.replicaSetSize))
 
 				// Since disruptionAllowed starts out false, if an eviction is ever allowed,
 				// that means the controller is working.
-				err = wait.PollImmediate(framework.Poll, timeout, func() (bool, error) {
-					err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+				err = wait.PollImmediateWithContext(ctx, framework.Poll, timeout, func(ctx context.Context) (bool, error) {
+					err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 					if err != nil {
 						return false, nil
 					}
@@ -343,15 +346,15 @@ var _ = SIGDescribe("DisruptionController", func() {
 		Testname: PodDisruptionBudget: block an eviction until the PDB is updated to allow it
 		Description: Eviction API must block an eviction until the PDB is updated to allow it
 	*/
-	framework.ConformanceIt("should block an eviction until the PDB is updated to allow it", func() {
+	framework.ConformanceIt("should block an eviction until the PDB is updated to allow it", func(ctx context.Context) {
 		ginkgo.By("Creating a pdb that targets all three pods in a test replica set")
-		createPDBMinAvailableOrDie(cs, ns, defaultName, intstr.FromInt(3), defaultLabels)
-		createReplicaSetOrDie(cs, ns, 3, false)
+		createPDBMinAvailableOrDie(ctx, cs, ns, defaultName, intstr.FromInt(3), defaultLabels)
+		createReplicaSetOrDie(ctx, cs, ns, 3, false)
 
 		ginkgo.By("First trying to evict a pod which shouldn't be evictable")
-		waitForPodsOrDie(cs, ns, 3) // make sure that they are running and so would be evictable with a different pdb
+		waitForPodsOrDie(ctx, cs, ns, 3) // make sure that they are running and so would be evictable with a different pdb
 
-		pod, err := locateRunningPod(cs, ns)
+		pod, err := locateRunningPod(ctx, cs, ns)
 		framework.ExpectNoError(err)
 		e := &policyv1.Eviction{
 			ObjectMeta: metav1.ObjectMeta{
@@ -359,25 +362,27 @@ var _ = SIGDescribe("DisruptionController", func() {
 				Namespace: ns,
 			},
 		}
-		err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+		err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 		framework.ExpectError(err, "pod eviction should fail")
-		framework.ExpectEqual(apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause), true, "pod eviction should fail with DisruptionBudget cause")
+		if !apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause) {
+			framework.Failf("pod eviction should fail with DisruptionBudget cause. The error was \"%v\"", err)
+		}
 
 		ginkgo.By("Updating the pdb to allow a pod to be evicted")
-		updatePDBOrDie(cs, ns, defaultName, func(pdb *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
+		updatePDBOrDie(ctx, cs, ns, defaultName, func(pdb *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget {
 			newMinAvailable := intstr.FromInt(2)
 			pdb.Spec.MinAvailable = &newMinAvailable
 			return pdb
 		}, cs.PolicyV1().PodDisruptionBudgets(ns).Update)
 
 		ginkgo.By("Trying to evict the same pod we tried earlier which should now be evictable")
-		waitForPodsOrDie(cs, ns, 3)
-		waitForPdbToObserveHealthyPods(cs, ns, 3)
-		err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+		waitForPodsOrDie(ctx, cs, ns, 3)
+		waitForPdbToObserveHealthyPods(ctx, cs, ns, 3)
+		err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 		framework.ExpectNoError(err) // the eviction is now allowed
 
 		ginkgo.By("Patching the pdb to disallow a pod to be evicted")
-		patchPDBOrDie(cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
+		patchPDBOrDie(ctx, cs, dc, ns, defaultName, func(old *policyv1.PodDisruptionBudget) (bytes []byte, err error) {
 			oldData, err := json.Marshal(old)
 			framework.ExpectNoError(err, "failed to marshal JSON for old data")
 			old.Spec.MinAvailable = nil
@@ -388,8 +393,8 @@ var _ = SIGDescribe("DisruptionController", func() {
 			return jsonpatch.CreateMergePatch(oldData, newData)
 		})
 
-		waitForPodsOrDie(cs, ns, 3)
-		pod, err = locateRunningPod(cs, ns) // locate a new running pod
+		waitForPodsOrDie(ctx, cs, ns, 3)
+		pod, err = locateRunningPod(ctx, cs, ns) // locate a new running pod
 		framework.ExpectNoError(err)
 		e = &policyv1.Eviction{
 			ObjectMeta: metav1.ObjectMeta{
@@ -397,22 +402,24 @@ var _ = SIGDescribe("DisruptionController", func() {
 				Namespace: ns,
 			},
 		}
-		err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+		err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 		framework.ExpectError(err, "pod eviction should fail")
-		framework.ExpectEqual(apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause), true, "pod eviction should fail with DisruptionBudget cause")
+		if !apierrors.HasStatusCause(err, policyv1.DisruptionBudgetCause) {
+			framework.Failf("pod eviction should fail with DisruptionBudget cause. The error was \"%v\"", err)
+		}
 
 		ginkgo.By("Deleting the pdb to allow a pod to be evicted")
-		deletePDBOrDie(cs, ns, defaultName)
+		deletePDBOrDie(ctx, cs, ns, defaultName)
 
 		ginkgo.By("Trying to evict the same pod we tried earlier which should now be evictable")
-		waitForPodsOrDie(cs, ns, 3)
-		err = cs.CoreV1().Pods(ns).EvictV1(context.TODO(), e)
+		waitForPodsOrDie(ctx, cs, ns, 3)
+		err = cs.CoreV1().Pods(ns).EvictV1(ctx, e)
 		framework.ExpectNoError(err) // the eviction is now allowed
 	})
 
 })
 
-func createPDBMinAvailableOrDie(cs kubernetes.Interface, ns string, name string, minAvailable intstr.IntOrString, labels map[string]string) {
+func createPDBMinAvailableOrDie(ctx context.Context, cs kubernetes.Interface, ns string, name string, minAvailable intstr.IntOrString, labels map[string]string) {
 	pdb := policyv1.PodDisruptionBudget{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
@@ -424,12 +431,12 @@ func createPDBMinAvailableOrDie(cs kubernetes.Interface, ns string, name string,
 			MinAvailable: &minAvailable,
 		},
 	}
-	_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Create(context.TODO(), &pdb, metav1.CreateOptions{})
+	_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Create(ctx, &pdb, metav1.CreateOptions{})
 	framework.ExpectNoError(err, "Waiting for the pdb to be created with minAvailable %d in namespace %s", minAvailable.IntVal, ns)
-	waitForPdbToBeProcessed(cs, ns, name)
+	waitForPdbToBeProcessed(ctx, cs, ns, name)
 }
 
-func createPDBMaxUnavailableOrDie(cs kubernetes.Interface, ns string, name string, maxUnavailable intstr.IntOrString) {
+func createPDBMaxUnavailableOrDie(ctx context.Context, cs kubernetes.Interface, ns string, name string, maxUnavailable intstr.IntOrString) {
 	pdb := policyv1.PodDisruptionBudget{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
@@ -440,39 +447,39 @@ func createPDBMaxUnavailableOrDie(cs kubernetes.Interface, ns string, name strin
 			MaxUnavailable: &maxUnavailable,
 		},
 	}
-	_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Create(context.TODO(), &pdb, metav1.CreateOptions{})
+	_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Create(ctx, &pdb, metav1.CreateOptions{})
 	framework.ExpectNoError(err, "Waiting for the pdb to be created with maxUnavailable %d in namespace %s", maxUnavailable.IntVal, ns)
-	waitForPdbToBeProcessed(cs, ns, name)
+	waitForPdbToBeProcessed(ctx, cs, ns, name)
 }
 
 type updateFunc func(pdb *policyv1.PodDisruptionBudget) *policyv1.PodDisruptionBudget
 type updateRestAPI func(ctx context.Context, podDisruptionBudget *policyv1.PodDisruptionBudget, opts metav1.UpdateOptions) (*policyv1.PodDisruptionBudget, error)
 type patchFunc func(pdb *policyv1.PodDisruptionBudget) ([]byte, error)
 
-func updatePDBOrDie(cs kubernetes.Interface, ns string, name string, f updateFunc, api updateRestAPI) (updated *policyv1.PodDisruptionBudget) {
+func updatePDBOrDie(ctx context.Context, cs kubernetes.Interface, ns string, name string, f updateFunc, api updateRestAPI) (updated *policyv1.PodDisruptionBudget) {
 	err := retry.RetryOnConflict(retry.DefaultRetry, func() error {
-		old, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), name, metav1.GetOptions{})
+		old, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, name, metav1.GetOptions{})
 		if err != nil {
 			return err
 		}
 		old = f(old)
-		if updated, err = api(context.TODO(), old, metav1.UpdateOptions{}); err != nil {
+		if updated, err = api(ctx, old, metav1.UpdateOptions{}); err != nil {
 			return err
 		}
 		return nil
 	})
 
 	framework.ExpectNoError(err, "Waiting for the PDB update to be processed in namespace %s", ns)
-	waitForPdbToBeProcessed(cs, ns, name)
+	waitForPdbToBeProcessed(ctx, cs, ns, name)
 	return updated
 }
 
-func patchPDBOrDie(cs kubernetes.Interface, dc dynamic.Interface, ns string, name string, f patchFunc, subresources ...string) (updated *policyv1.PodDisruptionBudget) {
+func patchPDBOrDie(ctx context.Context, cs kubernetes.Interface, dc dynamic.Interface, ns string, name string, f patchFunc, subresources ...string) (updated *policyv1.PodDisruptionBudget) {
 	err := retry.RetryOnConflict(retry.DefaultRetry, func() error {
-		old := getPDBStatusOrDie(dc, ns, name)
+		old := getPDBStatusOrDie(ctx, dc, ns, name)
 		patchBytes, err := f(old)
 		framework.ExpectNoError(err)
-		if updated, err = cs.PolicyV1().PodDisruptionBudgets(ns).Patch(context.TODO(), old.Name, types.MergePatchType, patchBytes, metav1.PatchOptions{}, subresources...); err != nil {
+		if updated, err = cs.PolicyV1().PodDisruptionBudgets(ns).Patch(ctx, old.Name, types.MergePatchType, patchBytes, metav1.PatchOptions{}, subresources...); err != nil {
 			return err
 		}
 		framework.ExpectNoError(err)
@@ -480,18 +487,18 @@ func patchPDBOrDie(cs kubernetes.Interface, dc dynamic.Interface, ns string, nam
 	})
 
 	framework.ExpectNoError(err, "Waiting for the pdb update to be processed in namespace %s", ns)
-	waitForPdbToBeProcessed(cs, ns, name)
+	waitForPdbToBeProcessed(ctx, cs, ns, name)
 	return updated
 }
 
-func deletePDBOrDie(cs kubernetes.Interface, ns string, name string) {
-	err := cs.PolicyV1().PodDisruptionBudgets(ns).Delete(context.TODO(), name, metav1.DeleteOptions{})
+func deletePDBOrDie(ctx context.Context, cs kubernetes.Interface, ns string, name string) {
+	err := cs.PolicyV1().PodDisruptionBudgets(ns).Delete(ctx, name, metav1.DeleteOptions{})
 	framework.ExpectNoError(err, "Deleting pdb in namespace %s", ns)
-	waitForPdbToBeDeleted(cs, ns, name)
+	waitForPdbToBeDeleted(ctx, cs, ns, name)
 }
 
-func listPDBs(cs kubernetes.Interface, ns string, labelSelector string, count int, expectedPDBNames []string) {
-	pdbList, err := cs.PolicyV1().PodDisruptionBudgets(ns).List(context.TODO(), metav1.ListOptions{LabelSelector: labelSelector})
+func listPDBs(ctx context.Context, cs kubernetes.Interface, ns string, labelSelector string, count int, expectedPDBNames []string) {
+	pdbList, err := cs.PolicyV1().PodDisruptionBudgets(ns).List(ctx, metav1.ListOptions{LabelSelector: labelSelector})
 	framework.ExpectNoError(err, "Listing PDB set in namespace %s", ns)
 	framework.ExpectEqual(len(pdbList.Items), count, "Expecting %d PDBs returned in namespace %s", count, ns)
 
@@ -502,18 +509,18 @@ func listPDBs(cs kubernetes.Interface, ns string, labelSelector string, count in
 	framework.ExpectConsistOf(pdbNames, expectedPDBNames, "Expecting returned PDBs '%s' in namespace %s", expectedPDBNames, ns)
 }
 
-func deletePDBCollection(cs kubernetes.Interface, ns string) {
+func deletePDBCollection(ctx context.Context, cs kubernetes.Interface, ns string) {
 	ginkgo.By("deleting a collection of PDBs")
-	err := cs.PolicyV1().PodDisruptionBudgets(ns).DeleteCollection(context.TODO(), metav1.DeleteOptions{}, metav1.ListOptions{})
+	err := cs.PolicyV1().PodDisruptionBudgets(ns).DeleteCollection(ctx, metav1.DeleteOptions{}, metav1.ListOptions{})
 	framework.ExpectNoError(err, "Deleting PDB set in namespace %s", ns)
 
-	waitForPDBCollectionToBeDeleted(cs, ns)
+	waitForPDBCollectionToBeDeleted(ctx, cs, ns)
 }
 
-func waitForPDBCollectionToBeDeleted(cs kubernetes.Interface, ns string) {
+func waitForPDBCollectionToBeDeleted(ctx context.Context, cs kubernetes.Interface, ns string) {
 	ginkgo.By("Waiting for the PDB collection to be deleted")
-	err := wait.PollImmediate(framework.Poll, schedulingTimeout, func() (bool, error) {
-		pdbList, err := cs.PolicyV1().PodDisruptionBudgets(ns).List(context.TODO(), metav1.ListOptions{})
+	err := wait.PollImmediateWithContext(ctx, framework.Poll, schedulingTimeout, func(ctx context.Context) (bool, error) {
+		pdbList, err := cs.PolicyV1().PodDisruptionBudgets(ns).List(ctx, metav1.ListOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -525,7 +532,7 @@ func waitForPDBCollectionToBeDeleted(cs kubernetes.Interface, ns string) {
 	framework.ExpectNoError(err, "Waiting for the PDB collection to be deleted in namespace %s", ns)
 }
 
-func createPodsOrDie(cs kubernetes.Interface, ns string, n int) {
+func createPodsOrDie(ctx context.Context, cs kubernetes.Interface, ns string, n int) {
 	for i := 0; i < n; i++ {
 		pod := &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
@@ -544,15 +551,15 @@ func createPodsOrDie(cs kubernetes.Interface, ns string, n int) {
 			},
 		}
 
-		_, err := cs.CoreV1().Pods(ns).Create(context.TODO(), pod, metav1.CreateOptions{})
+		_, err := cs.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{})
 		framework.ExpectNoError(err, "Creating pod %q in namespace %q", pod.Name, ns)
 	}
 }
 
-func waitForPodsOrDie(cs kubernetes.Interface, ns string, n int) {
+func waitForPodsOrDie(ctx context.Context, cs kubernetes.Interface, ns string, n int) {
 	ginkgo.By("Waiting for all pods to be running")
-	err := wait.PollImmediate(framework.Poll, schedulingTimeout, func() (bool, error) {
-		pods, err := cs.CoreV1().Pods(ns).List(context.TODO(), metav1.ListOptions{LabelSelector: "foo=bar"})
+	err := wait.PollImmediateWithContext(ctx, framework.Poll, schedulingTimeout, func(ctx context.Context) (bool, error) {
+		pods, err := cs.CoreV1().Pods(ns).List(ctx, metav1.ListOptions{LabelSelector: "foo=bar"})
 		if err != nil {
 			return false, err
 		}
@@ -579,7 +586,7 @@ func waitForPodsOrDie(cs kubernetes.Interface, ns string, n int) {
 	framework.ExpectNoError(err, "Waiting for pods in namespace %q to be ready", ns)
 }
 
-func createReplicaSetOrDie(cs kubernetes.Interface, ns string, size int32, exclusive bool) {
+func createReplicaSetOrDie(ctx context.Context, cs kubernetes.Interface, ns string, size int32, exclusive bool) {
 	container := v1.Container{
 		Name:  "donothing",
 		Image: imageutils.GetPauseImageName(),
@@ -611,14 +618,14 @@ func createReplicaSetOrDie(cs kubernetes.Interface, ns string, size int32, exclu
 		},
 	}
 
-	_, err := cs.AppsV1().ReplicaSets(ns).Create(context.TODO(), rs, metav1.CreateOptions{})
+	_, err := cs.AppsV1().ReplicaSets(ns).Create(ctx, rs, metav1.CreateOptions{})
 	framework.ExpectNoError(err, "Creating replica set %q in namespace %q", rs.Name, ns)
 }
 
-func locateRunningPod(cs kubernetes.Interface, ns string) (pod *v1.Pod, err error) {
+func locateRunningPod(ctx context.Context, cs kubernetes.Interface, ns string) (pod *v1.Pod, err error) {
 	ginkgo.By("locating a running pod")
-	err = wait.PollImmediate(framework.Poll, schedulingTimeout, func() (bool, error) {
-		podList, err := cs.CoreV1().Pods(ns).List(context.TODO(), metav1.ListOptions{})
+	err = wait.PollImmediateWithContext(ctx, framework.Poll, schedulingTimeout, func(ctx context.Context) (bool, error) {
+		podList, err := cs.CoreV1().Pods(ns).List(ctx, metav1.ListOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -636,10 +643,10 @@ func locateRunningPod(cs kubernetes.Interface, ns string) (pod *v1.Pod, err erro
 	return pod, err
 }
 
-func waitForPdbToBeProcessed(cs kubernetes.Interface, ns string, name string) {
+func waitForPdbToBeProcessed(ctx context.Context, cs kubernetes.Interface, ns string, name string) {
 	ginkgo.By("Waiting for the pdb to be processed")
-	err := wait.PollImmediate(framework.Poll, schedulingTimeout, func() (bool, error) {
-		pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), name, metav1.GetOptions{})
+	err := wait.PollImmediateWithContext(ctx, framework.Poll, schedulingTimeout, func(ctx context.Context) (bool, error) {
+		pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, name, metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -651,10 +658,10 @@ func waitForPdbToBeProcessed(cs kubernetes.Interface, ns string, name string) {
 	framework.ExpectNoError(err, "Waiting for the pdb to be processed in namespace %s", ns)
 }
 
-func waitForPdbToBeDeleted(cs kubernetes.Interface, ns string, name string) {
+func waitForPdbToBeDeleted(ctx context.Context, cs kubernetes.Interface, ns string, name string) {
 	ginkgo.By("Waiting for the pdb to be deleted")
-	err := wait.PollImmediate(framework.Poll, schedulingTimeout, func() (bool, error) {
-		_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), name, metav1.GetOptions{})
+	err := wait.PollImmediateWithContext(ctx, framework.Poll, schedulingTimeout, func(ctx context.Context) (bool, error) {
+		_, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, name, metav1.GetOptions{})
 		if apierrors.IsNotFound(err) {
 			return true, nil // done
 		}
@@ -666,10 +673,10 @@ func waitForPdbToBeDeleted(cs kubernetes.Interface, ns string, name string) {
 	framework.ExpectNoError(err, "Waiting for the pdb to be deleted in namespace %s", ns)
 }
 
-func waitForPdbToObserveHealthyPods(cs kubernetes.Interface, ns string, healthyCount int32) {
+func waitForPdbToObserveHealthyPods(ctx context.Context, cs kubernetes.Interface, ns string, healthyCount int32) {
 	ginkgo.By("Waiting for the pdb to observed all healthy pods")
-	err := wait.PollImmediate(framework.Poll, wait.ForeverTestTimeout, func() (bool, error) {
-		pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(context.TODO(), "foo", metav1.GetOptions{})
+	err := wait.PollImmediateWithContext(ctx, framework.Poll, wait.ForeverTestTimeout, func(ctx context.Context) (bool, error) {
+		pdb, err := cs.PolicyV1().PodDisruptionBudgets(ns).Get(ctx, "foo", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -681,9 +688,9 @@ func waitForPdbToObserveHealthyPods(cs kubernetes.Interface, ns string, healthyC
 	framework.ExpectNoError(err, "Waiting for the pdb in namespace %s to observed %d healthy pods", ns, healthyCount)
 }
 
-func getPDBStatusOrDie(dc dynamic.Interface, ns string, name string) *policyv1.PodDisruptionBudget {
+func getPDBStatusOrDie(ctx context.Context, dc dynamic.Interface, ns string, name string) *policyv1.PodDisruptionBudget {
 	pdbStatusResource := policyv1.SchemeGroupVersion.WithResource("poddisruptionbudgets")
-	unstruct, err := dc.Resource(pdbStatusResource).Namespace(ns).Get(context.TODO(), name, metav1.GetOptions{}, "status")
+	unstruct, err := dc.Resource(pdbStatusResource).Namespace(ns).Get(ctx, name, metav1.GetOptions{}, "status")
 	framework.ExpectNoError(err)
 	pdb, err := unstructuredToPDB(unstruct)
 	framework.ExpectNoError(err, "Getting the status of the pdb %s in namespace %s", name, ns)
