@@ -20,7 +20,7 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 
 	v1 "k8s.io/api/core/v1"
@@ -35,7 +35,6 @@ import (
 	e2eskipper "k8s.io/kubernetes/test/e2e/framework/skipper"
 	e2evolume "k8s.io/kubernetes/test/e2e/framework/volume"
 	storageframework "k8s.io/kubernetes/test/e2e/storage/framework"
-	storageutils "k8s.io/kubernetes/test/e2e/storage/utils"
 	admissionapi "k8s.io/pod-security-admission/api"
 )
 
@@ -104,13 +103,13 @@ func (p *ephemeralTestSuite) SkipUnsupportedTests(driver storageframework.TestDr
 
 func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pattern storageframework.TestPattern) {
 	type local struct {
-		config        *storageframework.PerTestConfig
-		driverCleanup func()
+		config *storageframework.PerTestConfig
 
 		testCase *EphemeralTest
 		resource *storageframework.VolumeResource
 	}
 	var (
+		dInfo   = driver.GetDriverInfo()
 		eDriver storageframework.EphemeralTestDriver
 		l       local
 	)
@@ -118,9 +117,9 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 	// Beware that it also registers an AfterEach which renders f unusable. Any code using
 	// f must run inside an It or Context callback.
 	f := framework.NewFrameworkWithCustomTimeouts("ephemeral", storageframework.GetDriverTimeouts(driver))
-	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
+	f.NamespacePodSecurityLevel = admissionapi.LevelPrivileged
 
-	init := func() {
+	init := func(ctx context.Context) {
 		if pattern.VolType == storageframework.CSIInlineVolume {
 			eDriver, _ = driver.(storageframework.EphemeralTestDriver)
 		}
@@ -128,18 +127,27 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 			// The GenericEphemeralVolume feature is GA, but
 			// perhaps this test is run against an older Kubernetes
 			// where the feature might be disabled.
-			enabled, err := GenericEphemeralVolumesEnabled(f.ClientSet, f.Timeouts, f.Namespace.Name)
+			enabled, err := GenericEphemeralVolumesEnabled(ctx, f.ClientSet, f.Timeouts, f.Namespace.Name)
 			framework.ExpectNoError(err, "check GenericEphemeralVolume feature")
 			if !enabled {
 				e2eskipper.Skipf("Cluster doesn't support %q volumes -- skipping", pattern.VolType)
 			}
 		}
+		// A driver might support the Topology capability which is incompatible with the VolumeBindingMode immediate because
+		// volumes might be provisioned immediately in a different zone to where the workload is located.
+		if pattern.BindingMode == storagev1.VolumeBindingImmediate && len(dInfo.TopologyKeys) > 0 {
+			e2eskipper.Skipf("VolumeBindingMode immediate is not compatible with a multi-topology environment.")
+		}
 
 		l = local{}
 
+		if !driver.GetDriverInfo().Capabilities[storageframework.CapOnlineExpansion] {
+			pattern.AllowExpansion = false
+		}
+
 		// Now do the more expensive test initialization.
-		l.config, l.driverCleanup = driver.PrepareTest(f)
-		l.resource = storageframework.CreateVolumeResource(driver, l.config, pattern, e2evolume.SizeRange{})
+		l.config = driver.PrepareTest(ctx, f)
+		l.resource = storageframework.CreateVolumeResource(ctx, driver, l.config, pattern, e2evolume.SizeRange{})
 
 		switch pattern.VolType {
 		case storageframework.CSIInlineVolume:
@@ -164,24 +172,23 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 		}
 	}
 
-	cleanup := func() {
+	cleanup := func(ctx context.Context) {
 		var cleanUpErrs []error
-		cleanUpErrs = append(cleanUpErrs, l.resource.CleanupResource())
-		cleanUpErrs = append(cleanUpErrs, storageutils.TryFunc(l.driverCleanup))
+		cleanUpErrs = append(cleanUpErrs, l.resource.CleanupResource(ctx))
 		err := utilerrors.NewAggregate(cleanUpErrs)
 		framework.ExpectNoError(err, "while cleaning up")
 	}
 
-	ginkgo.It("should create read-only inline ephemeral volume", func() {
+	ginkgo.It("should create read-only inline ephemeral volume", func(ctx context.Context) {
 		if pattern.VolMode == v1.PersistentVolumeBlock {
 			e2eskipper.Skipf("raw block volumes cannot be read-only")
 		}
 
-		init()
-		defer cleanup()
+		init(ctx)
+		ginkgo.DeferCleanup(cleanup)
 
 		l.testCase.ReadOnly = true
-		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
+		l.testCase.RunningPodCheck = func(ctx context.Context, pod *v1.Pod) interface{} {
 			command := "mount | grep /mnt/test | grep ro,"
 			if framework.NodeOSDistroIs("windows") {
 				// attempt to create a dummy file and expect for it not to be created
@@ -190,15 +197,15 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 			e2evolume.VerifyExecInPodSucceed(f, pod, command)
 			return nil
 		}
-		l.testCase.TestEphemeral()
+		l.testCase.TestEphemeral(ctx)
 	})
 
-	ginkgo.It("should create read/write inline ephemeral volume", func() {
-		init()
-		defer cleanup()
+	ginkgo.It("should create read/write inline ephemeral volume", func(ctx context.Context) {
+		init(ctx)
+		ginkgo.DeferCleanup(cleanup)
 
 		l.testCase.ReadOnly = false
-		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
+		l.testCase.RunningPodCheck = func(ctx context.Context, pod *v1.Pod) interface{} {
 			command := "mount | grep /mnt/test | grep rw,"
 			if framework.NodeOSDistroIs("windows") {
 				// attempt to create a dummy file and expect for it to be created
@@ -210,12 +217,73 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 			e2evolume.VerifyExecInPodSucceed(f, pod, command)
 			return nil
 		}
-		l.testCase.TestEphemeral()
+		l.testCase.TestEphemeral(ctx)
 	})
 
-	ginkgo.It("should support two pods which have the same volume definition", func() {
-		init()
-		defer cleanup()
+	ginkgo.It("should support expansion of pvcs created for ephemeral pvcs", func(ctx context.Context) {
+		if pattern.VolType != storageframework.GenericEphemeralVolume {
+			e2eskipper.Skipf("Skipping %s test for expansion", pattern.VolType)
+		}
+
+		init(ctx)
+		ginkgo.DeferCleanup(cleanup)
+
+		if !driver.GetDriverInfo().Capabilities[storageframework.CapOnlineExpansion] {
+			e2eskipper.Skipf("Driver %q does not support online volume expansion - skipping", driver.GetDriverInfo().Name)
+		}
+
+		l.testCase.ReadOnly = false
+		l.testCase.RunningPodCheck = func(ctx context.Context, pod *v1.Pod) interface{} {
+			podName := pod.Name
+			framework.Logf("Running volume expansion checks %s", podName)
+
+			outerPodVolumeSpecName := ""
+			for i := range pod.Spec.Volumes {
+				volume := pod.Spec.Volumes[i]
+				if volume.Ephemeral != nil {
+					outerPodVolumeSpecName = volume.Name
+					break
+				}
+			}
+			pvcName := fmt.Sprintf("%s-%s", podName, outerPodVolumeSpecName)
+			pvc, err := f.ClientSet.CoreV1().PersistentVolumeClaims(pod.Namespace).Get(ctx, pvcName, metav1.GetOptions{})
+			framework.ExpectNoError(err, "error getting ephemeral pvc")
+
+			ginkgo.By("Expanding current pvc")
+			currentPvcSize := pvc.Spec.Resources.Requests[v1.ResourceStorage]
+			newSize := currentPvcSize.DeepCopy()
+			newSize.Add(resource.MustParse("1Gi"))
+			framework.Logf("currentPvcSize %s, requested new size %s", currentPvcSize.String(), newSize.String())
+
+			newPVC, err := ExpandPVCSize(ctx, pvc, newSize, f.ClientSet)
+			framework.ExpectNoError(err, "While updating pvc for more size")
+			pvc = newPVC
+			gomega.Expect(pvc).NotTo(gomega.BeNil())
+
+			pvcSize := pvc.Spec.Resources.Requests[v1.ResourceStorage]
+			if pvcSize.Cmp(newSize) != 0 {
+				framework.Failf("error updating pvc %s from %s to %s size", pvc.Name, currentPvcSize.String(), newSize.String())
+			}
+
+			ginkgo.By("Waiting for cloudprovider resize to finish")
+			err = WaitForControllerVolumeResize(ctx, pvc, f.ClientSet, totalResizeWaitPeriod)
+			framework.ExpectNoError(err, "While waiting for pvc resize to finish")
+
+			ginkgo.By("Waiting for file system resize to finish")
+			pvc, err = WaitForFSResize(ctx, pvc, f.ClientSet)
+			framework.ExpectNoError(err, "while waiting for fs resize to finish")
+
+			pvcConditions := pvc.Status.Conditions
+			framework.ExpectEqual(len(pvcConditions), 0, "pvc should not have conditions")
+			return nil
+		}
+		l.testCase.TestEphemeral(ctx)
+
+	})
+
+	ginkgo.It("should support two pods which have the same volume definition", func(ctx context.Context) {
+		init(ctx)
+		ginkgo.DeferCleanup(cleanup)
 
 		// We test in read-only mode if that is all that the driver supports,
 		// otherwise read/write. For PVC, both are assumed to be false.
@@ -225,13 +293,13 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 			_, shared, readOnly = eDriver.GetVolume(l.config, 0)
 		}
 
-		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
+		l.testCase.RunningPodCheck = func(ctx context.Context, pod *v1.Pod) interface{} {
 			// Create another pod with the same inline volume attributes.
-			pod2 := StartInPodWithInlineVolume(f.ClientSet, f.Namespace.Name, "inline-volume-tester2", "sleep 100000",
+			pod2 := StartInPodWithInlineVolume(ctx, f.ClientSet, f.Namespace.Name, "inline-volume-tester2", "sleep 100000",
 				[]v1.VolumeSource{pod.Spec.Volumes[0].VolumeSource},
 				readOnly,
 				l.testCase.Node)
-			framework.ExpectNoError(e2epod.WaitTimeoutForPodRunningInNamespace(f.ClientSet, pod2.Name, pod2.Namespace, f.Timeouts.PodStartSlow), "waiting for second pod with inline volume")
+			framework.ExpectNoError(e2epod.WaitTimeoutForPodRunningInNamespace(ctx, f.ClientSet, pod2.Name, pod2.Namespace, f.Timeouts.PodStartSlow), "waiting for second pod with inline volume")
 
 			// If (and only if) we were able to mount
 			// read/write and volume data is not shared
@@ -244,24 +312,28 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 				e2evolume.VerifyExecInPodSucceed(f, pod2, "[ ! -f /mnt/test-0/hello-world ]")
 			}
 
-			defer StopPodAndDependents(f.ClientSet, f.Timeouts, pod2)
+			// TestEphemeral expects the pod to be fully deleted
+			// when this function returns, so don't delay this
+			// cleanup.
+			StopPodAndDependents(ctx, f.ClientSet, f.Timeouts, pod2)
+
 			return nil
 		}
 
-		l.testCase.TestEphemeral()
+		l.testCase.TestEphemeral(ctx)
 	})
 
-	ginkgo.It("should support multiple inline ephemeral volumes", func() {
+	ginkgo.It("should support multiple inline ephemeral volumes", func(ctx context.Context) {
 		if pattern.BindingMode == storagev1.VolumeBindingImmediate &&
 			pattern.VolType == storageframework.GenericEphemeralVolume {
 			e2eskipper.Skipf("Multiple generic ephemeral volumes with immediate binding may cause pod startup failures when the volumes get created in separate topology segments.")
 		}
 
-		init()
-		defer cleanup()
+		init(ctx)
+		ginkgo.DeferCleanup(cleanup)
 
 		l.testCase.NumInlineVolumes = 2
-		l.testCase.TestEphemeral()
+		l.testCase.TestEphemeral(ctx)
 	})
 }
 
@@ -291,7 +363,7 @@ type EphemeralTest struct {
 	// RunningPodCheck is invoked while a pod using an inline volume is running.
 	// It can execute additional checks on the pod and its volume(s). Any data
 	// returned by it is passed to StoppedPodCheck.
-	RunningPodCheck func(pod *v1.Pod) interface{}
+	RunningPodCheck func(ctx context.Context, pod *v1.Pod) interface{}
 
 	// StoppedPodCheck is invoked after ensuring that the pod is gone.
 	// It is passed the data gather by RunningPodCheck or nil if that
@@ -299,7 +371,7 @@ type EphemeralTest struct {
 	// like for example verifying that the ephemeral volume was really
 	// removed. How to do such a check is driver-specific and not
 	// covered by the generic storage test suite.
-	StoppedPodCheck func(nodeName string, runningPodData interface{})
+	StoppedPodCheck func(ctx context.Context, nodeName string, runningPodData interface{})
 
 	// NumInlineVolumes sets the number of ephemeral inline volumes per pod.
 	// Unset (= zero) is the same as one.
@@ -310,7 +382,7 @@ type EphemeralTest struct {
 }
 
 // TestEphemeral tests pod creation with one ephemeral volume.
-func (t EphemeralTest) TestEphemeral() {
+func (t EphemeralTest) TestEphemeral(ctx context.Context) {
 	client := t.Client
 	gomega.Expect(client).NotTo(gomega.BeNil(), "EphemeralTest.Client is required")
 
@@ -343,39 +415,39 @@ func (t EphemeralTest) TestEphemeral() {
 		}
 		volumes = append(volumes, volume)
 	}
-	pod := StartInPodWithInlineVolume(client, t.Namespace, "inline-volume-tester", command, volumes, t.ReadOnly, t.Node)
+	pod := StartInPodWithInlineVolume(ctx, client, t.Namespace, "inline-volume-tester", command, volumes, t.ReadOnly, t.Node)
 	defer func() {
 		// pod might be nil now.
-		StopPodAndDependents(client, t.Timeouts, pod)
+		StopPodAndDependents(ctx, client, t.Timeouts, pod)
 	}()
-	framework.ExpectNoError(e2epod.WaitTimeoutForPodRunningInNamespace(client, pod.Name, pod.Namespace, t.Timeouts.PodStartSlow), "waiting for pod with inline volume")
-	runningPod, err := client.CoreV1().Pods(pod.Namespace).Get(context.TODO(), pod.Name, metav1.GetOptions{})
+	framework.ExpectNoError(e2epod.WaitTimeoutForPodRunningInNamespace(ctx, client, pod.Name, pod.Namespace, t.Timeouts.PodStartSlow), "waiting for pod with inline volume")
+	runningPod, err := client.CoreV1().Pods(pod.Namespace).Get(ctx, pod.Name, metav1.GetOptions{})
 	framework.ExpectNoError(err, "get pod")
 	actualNodeName := runningPod.Spec.NodeName
 
 	// Run the checker of the running pod.
 	var runningPodData interface{}
 	if t.RunningPodCheck != nil {
-		runningPodData = t.RunningPodCheck(pod)
+		runningPodData = t.RunningPodCheck(ctx, pod)
 	}
 
-	StopPodAndDependents(client, t.Timeouts, pod)
+	StopPodAndDependents(ctx, client, t.Timeouts, pod)
 	pod = nil // Don't stop twice.
 
 	// There should be no dangling PVCs in the namespace now. There might be for
 	// generic ephemeral volumes, if something went wrong...
-	pvcs, err := client.CoreV1().PersistentVolumeClaims(t.Namespace).List(context.TODO(), metav1.ListOptions{})
+	pvcs, err := client.CoreV1().PersistentVolumeClaims(t.Namespace).List(ctx, metav1.ListOptions{})
 	framework.ExpectNoError(err, "list PVCs")
 	gomega.Expect(pvcs.Items).Should(gomega.BeEmpty(), "no dangling PVCs")
 
 	if t.StoppedPodCheck != nil {
-		t.StoppedPodCheck(actualNodeName, runningPodData)
+		t.StoppedPodCheck(ctx, actualNodeName, runningPodData)
 	}
 }
 
 // StartInPodWithInlineVolume starts a command in a pod with given volume(s) mounted to /mnt/test-<number> directory.
 // The caller is responsible for checking the pod and deleting it.
-func StartInPodWithInlineVolume(c clientset.Interface, ns, podName, command string, volumes []v1.VolumeSource, readOnly bool, node e2epod.NodeSelection) *v1.Pod {
+func StartInPodWithInlineVolume(ctx context.Context, c clientset.Interface, ns, podName, command string, volumes []v1.VolumeSource, readOnly bool, node e2epod.NodeSelection) *v1.Pod {
 	pod := &v1.Pod{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "Pod",
@@ -426,15 +498,15 @@ func StartInPodWithInlineVolume(c clientset.Interface, ns, podName, command stri
 			})
 	}
 
-	pod, err := c.CoreV1().Pods(ns).Create(context.TODO(), pod, metav1.CreateOptions{})
+	pod, err := c.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{})
 	framework.ExpectNoError(err, "failed to create pod")
 	return pod
 }
 
 // CSIInlineVolumesEnabled checks whether the running cluster has the CSIInlineVolumes feature gate enabled.
 // It does that by trying to create a pod that uses that feature.
-func CSIInlineVolumesEnabled(c clientset.Interface, t *framework.TimeoutContext, ns string) (bool, error) {
-	return VolumeSourceEnabled(c, t, ns, v1.VolumeSource{
+func CSIInlineVolumesEnabled(ctx context.Context, c clientset.Interface, t *framework.TimeoutContext, ns string) (bool, error) {
+	return VolumeSourceEnabled(ctx, c, t, ns, v1.VolumeSource{
 		CSI: &v1.CSIVolumeSource{
 			Driver: "no-such-driver.example.com",
 		},
@@ -443,9 +515,9 @@ func CSIInlineVolumesEnabled(c clientset.Interface, t *framework.TimeoutContext,
 
 // GenericEphemeralVolumesEnabled checks whether the running cluster has the GenericEphemeralVolume feature gate enabled.
 // It does that by trying to create a pod that uses that feature.
-func GenericEphemeralVolumesEnabled(c clientset.Interface, t *framework.TimeoutContext, ns string) (bool, error) {
+func GenericEphemeralVolumesEnabled(ctx context.Context, c clientset.Interface, t *framework.TimeoutContext, ns string) (bool, error) {
 	storageClassName := "no-such-storage-class"
-	return VolumeSourceEnabled(c, t, ns, v1.VolumeSource{
+	return VolumeSourceEnabled(ctx, c, t, ns, v1.VolumeSource{
 		Ephemeral: &v1.EphemeralVolumeSource{
 			VolumeClaimTemplate: &v1.PersistentVolumeClaimTemplate{
 				Spec: v1.PersistentVolumeClaimSpec{
@@ -464,7 +536,7 @@ func GenericEphemeralVolumesEnabled(c clientset.Interface, t *framework.TimeoutC
 
 // VolumeSourceEnabled checks whether a certain kind of volume source is enabled by trying
 // to create a pod that uses it.
-func VolumeSourceEnabled(c clientset.Interface, t *framework.TimeoutContext, ns string, volume v1.VolumeSource) (bool, error) {
+func VolumeSourceEnabled(ctx context.Context, c clientset.Interface, t *framework.TimeoutContext, ns string, volume v1.VolumeSource) (bool, error) {
 	pod := &v1.Pod{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "Pod",
@@ -496,12 +568,12 @@ func VolumeSourceEnabled(c clientset.Interface, t *framework.TimeoutContext, ns
 		},
 	}
 
-	pod, err := c.CoreV1().Pods(ns).Create(context.TODO(), pod, metav1.CreateOptions{})
+	pod, err := c.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{})
 
 	switch {
 	case err == nil:
 		// Pod was created, feature supported.
-		StopPodAndDependents(c, t, pod)
+		StopPodAndDependents(ctx, c, t, pod)
 		return true, nil
 	case apierrors.IsInvalid(err):
 		// "Invalid" because it uses a feature that isn't supported.
